@@ -4,26 +4,25 @@
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
 from __future__ import print_function
-from distutils.version import LooseVersion
 import errno
+import glob
 import json
 import logging
 import pipes
 import os
-import glob
-from functools import partial
-from os import path as osp
-from os.path import join as pjoin
-from tornado import gen
-from tornado.ioloop import IOLoop
-from subprocess import CalledProcessError, Popen, STDOUT
+import re
 import shutil
 import sys
 import tarfile
+from distutils.version import LooseVersion
+from functools import partial
 from jupyter_core.paths import ENV_JUPYTER_PATH, jupyter_config_path
-from notebook.nbextensions import (
-    GREEN_ENABLED, GREEN_OK, RED_DISABLED, RED_X
-)
+from notebook.nbextensions import GREEN_ENABLED, GREEN_OK, RED_DISABLED, RED_X
+from os import path as osp
+from os.path import join as pjoin
+from subprocess import CalledProcessError, Popen, STDOUT
+from tornado import gen
+from tornado.ioloop import IOLoop
 
 from .semver import Range, gte, lt, lte, gt
 from ._version import __version__
@@ -68,6 +67,7 @@ def run(cmd, **kwargs):
     kwargs.setdefault('shell', sys.platform == 'win32')
     kwargs.setdefault('env', os.environ)
     kwargs.setdefault('stderr', STDOUT)
+    proc = None
     yield gen.moment  # Sync up to the iterator
     try:
         proc = Popen(cmd, **kwargs)
@@ -83,7 +83,15 @@ def run(cmd, **kwargs):
         logger.info(output)
         raise error
     finally:
-        proc.wait()
+        if proc:
+            proc.wait()
+
+
+def watch(cwd, logger=None):
+    """Run watch mode in a given directory"""
+    loop = IOLoop.instance()
+    loop.add_callback(run, [get_npm_name(), 'run', 'watch'], cwd=cwd,
+        logger=logger)
 
 
 def install_extension(extension, app_dir=None, logger=None):
@@ -128,7 +136,7 @@ def install_extension_async(extension, app_dir=None, logger=None, abort_callback
         return
 
     _ensure_app_dirs(app_dir, logger)
-    
+
     target = pjoin(app_dir, 'extensions', 'temp')
     if os.path.exists(target):
         shutil.rmtree(target)
@@ -165,19 +173,6 @@ def install_extension_async(extension, app_dir=None, logger=None, abort_callback
         if osp.exists(path) and other['location'] == 'app':
             os.remove(path)
 
-    # Handle any schemas.
-    schemaDir = data['jupyterlab'].get('schemaDir', None)
-    if schemaDir:
-        dest = pjoin(app_dir, 'schemas')
-        _copy_tar_files(pjoin(target, fname), schemaDir, dest)
-
-    # Handle a theme.
-    themeDir = data['jupyterlab'].get('themeDir', None)
-    if themeDir:
-        normedName = data['name'].replace('@', '').replace('/', '')
-        dest = pjoin(app_dir, 'themes', normedName)
-        _copy_tar_files(pjoin(target, fname), themeDir, dest)
-
     # Remove an existing extension tarball.
     ext_path = pjoin(app_dir, 'extensions', fname)
     if os.path.exists(ext_path):
@@ -186,8 +181,8 @@ def install_extension_async(extension, app_dir=None, logger=None, abort_callback
     shutil.move(pjoin(target, fname), pjoin(app_dir, 'extensions'))
     shutil.rmtree(target)
 
-    # Remove any existing package from staging/node_modules
-    target = pjoin(app_dir, 'staging', 'node_modules', data['name'])
+    # Clear staging/node_modules
+    target = pjoin(app_dir, 'staging', 'node_modules')
     target = target.replace('/', os.sep)
     if os.path.exists(target):
         shutil.rmtree(target)
@@ -452,9 +447,10 @@ def list_extensions(app_dir=None, logger=None):
     if app:
         logger.info('   app dir: %s' % app_dir)
         for item in sorted(app):
+            logger.info(item)
             version = extensions[item]['version']
             extra = ''
-            if item in disabled:
+            if is_disabled(item, disabled):
                 extra += ' %s' % RED_DISABLED
             else:
                 extra += ' %s' % GREEN_ENABLED
@@ -462,7 +458,7 @@ def list_extensions(app_dir=None, logger=None):
                 extra += ' %s' % RED_X
             else:
                 extra += ' %s' % GREEN_OK
-            logger.info('        %s@%s%s' % (item, version, extra))
+            logger.info('        %s v%s%s' % (item, version, extra))
             if errors[item]:
                 msg = _format_compatibility_errors(item, version, errors[item])
                 logger.warn(msg + '\n')
@@ -483,7 +479,7 @@ def list_extensions(app_dir=None, logger=None):
                 extra += ' %s' % GREEN_OK
             if item in linked:
                 extra += '*'
-            logger.info('        %s@%s%s' % (item, version, extra))
+            logger.info('        %s v%s%s' % (item, version, extra))
             if errors[item]:
                 msg = _format_compatibility_errors(item, version, errors[item])
                 logger.warn(msg + '\n')
@@ -536,6 +532,13 @@ def build(app_dir=None, name=None, version=None, logger=None):
                    logger=logger)
     return IOLoop.instance().run_sync(func)
 
+def is_disabled(name, disabled=[]):
+    for pattern in disabled:
+        if name == pattern:
+            return True
+        if re.compile(pattern).match(name) != None:
+            return True
+    return False
 
 @gen.coroutine
 def build_async(app_dir=None, name=None, version=None, logger=None, abort_callback=None):
@@ -573,9 +576,10 @@ def build_async(app_dir=None, name=None, version=None, logger=None, abort_callba
     npm = get_npm_name()
 
     # Make sure packages are installed.
-    yield run([npm, 'install'], cwd=staging, logger=logger, abort_callback=abort_callback)
+    yield run([npm, 'install', '--no-optional'], cwd=staging, logger=logger, abort_callback=abort_callback)
 
     # Build the app.
+    yield run([npm, 'run', 'clean'], cwd=staging, logger=logger, abort_callback= abort_callback)
     yield run([npm, 'run', 'build'], cwd=staging, logger=logger, abort_callback= abort_callback)
 
     # Move the app to the static dir.
@@ -757,25 +761,18 @@ def _toggle_extension(extension, value, app_dir=None, logger=None):
     """Enable or disable a lab extension.
     """
     app_dir = get_app_dir(app_dir)
-    config = _get_page_config(app_dir)
     extensions = _get_extensions(app_dir)
-    core_extensions = _get_core_extensions()
-
-    if extension not in extensions and extension not in core_extensions:
-        raise ValueError('Extension %s is not installed' % extension)
+    config = _get_build_config(app_dir)
     disabled = config.get('disabledExtensions', [])
     if value and extension not in disabled:
         disabled.append(extension)
     if not value and extension in disabled:
         disabled.remove(extension)
-
-    # Prune extensions that are not installed.
-    disabled = [ext for ext in disabled
-                if (ext in extensions or ext in core_extensions)]
     config['disabledExtensions'] = disabled
     _write_page_config(config, app_dir, logger=logger)
 
 
+
 def _write_build_config(config, app_dir, logger):
     """Write the build config to the app dir.
     """
@@ -804,14 +801,13 @@ def _ensure_package(app_dir, logger=None, name=None, version=None):
     # Look for mismatched version.
     staging = pjoin(app_dir, 'staging')
     pkg_path = pjoin(staging, 'package.json')
-    version_updated = False
+
     if os.path.exists(pkg_path):
         with open(pkg_path) as fid:
             data = json.load(fid)
         if data['jupyterlab'].get('version', '') != version:
             shutil.rmtree(staging)
             os.makedirs(staging)
-            version_updated = True
 
     for fname in ['index.app.js', 'webpack.config.js']:
         dest = pjoin(staging, fname.replace('.app', ''))
@@ -819,34 +815,23 @@ def _ensure_package(app_dir, logger=None, name=None, version=None):
 
     # Template the package.json file.
     data = _get_package_template(app_dir, logger)
-    data['jupyterlab']['name'] = name or 'JupyterLab'
-    data['jupyterlab']['version'] = version
+
+    if version:
+        data['jupyterlab']['version'] = version
+
+    if name:
+        data['jupyterlab']['name'] = name
+
+    data['jupyterlab']['linkedPackages'] = _get_linked_packages(app_dir)
 
     pkg_path = pjoin(staging, 'package.json')
     with open(pkg_path, 'w') as fid:
         json.dump(data, fid, indent=4)
 
-    # Copy any missing or outdated schema or theme items.
-    for item in ['schemas', 'themes']:
-        local = pjoin(here, item)
-        if not os.path.exists(local):
-            os.makedirs(local)
-
-        for item_path in os.listdir(local):
-            src = pjoin(local, item_path)
-            dest = pjoin(app_dir, item, item_path)
-            if version_updated or not os.path.exists(dest):
-                if os.path.isdir(src):
-                    if os.path.exists(dest):
-                        shutil.rmtree(dest)
-                    shutil.copytree(src, dest)
-                else:
-                    shutil.copy(src, dest)
-
 
 def _ensure_app_dirs(app_dir, logger):
     """Ensure that the application directories exist"""
-    dirs = ['extensions', 'settings', 'schemas', 'themes', 'staging']
+    dirs = ['extensions', 'settings', 'staging', 'schemas', 'themes']
     for dname in dirs:
         path = pjoin(app_dir, dname)
         if not osp.exists(path):
